# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-17 13:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0011_auto_20170217_1120'),
    ]

    operations = [
        migrations.AlterField(
            model_name='incident',
            name='service',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='incidents', related_query_name='incident', to='statusboard.Service', verbose_name='service'),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django import forms
from django.forms import inlineformset_factory
from django.utils.translation import gettext_lazy as _

from .models import Incident
from .models import Service
from .models import ServiceGroup
from .models import IncidentUpdate
from .models import Maintenance
from .models import SERVICE_STATUSES


class ServiceGroupForm(forms.ModelForm):
    class Meta:
        model = ServiceGroup
        fields = ["name", "collapse", "priority"]


class ServiceForm(forms.ModelForm):
    class Meta:
        model = Service
        fields = [
            "name",
            "href",
            "description",
            "status",
            "priority",
            "groups",
        ]


class MaintenanceForm(forms.ModelForm):
    class Meta:
        model = Maintenance
        fields = ["name", "description", "scheduled"]


class IncidentForm(forms.ModelForm):
    service_status = forms.ChoiceField(
        choices=tuple([("", "---------")] + list(SERVICE_STATUSES)),
        label=_("Services status"),
        help_text=_("Update the status of involved services (an empty value means that they will be left unaltered)"),
        required=False,
    )

    class Meta:
        model = Incident
        fields = ["name", "occurred", "services", "service_status"]

    def __init__(self, *args, **kwargs):
        super(IncidentForm, self).__init__(*args, **kwargs)
        self.fields["services"].queryset = Service.objects.order_by("name")

    def save(self, commit=True):
        model = super(IncidentForm, self).save(commit=False)
        status = self.cleaned_data["service_status"]

        if commit:
            model.save()
            self.save_m2m()
            if status not in (None, ""):
                model.services.update(status=status)

        return model


IncidentUpdateFormSet = inlineformset_factory(Incident, IncidentUpdate, fields=("status", "description"), extra=1)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-16 11:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0008_servicegroup_collapse'),
    ]

    operations = [
        migrations.AddField(
            model_name='servicegroup',
            name='position',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.urls import re_path, include
from django.contrib.auth import views as auth_views

from rest_framework import routers

from . import views


router = routers.DefaultRouter()
router.register(r"servicegroup", views.ServiceGroupViewSet)
router.register(r"service", views.ServiceViewSet)
router.register(r"incident", views.IncidentViewSet)
router.register(r"incidentupdate", views.IncidentUpdateViewSet)
router.register(r"maintenance", views.MaintenanceViewSet)

app_name = "statusboard"

service_urls = [
    re_path("^create/$", views.ServiceCreate.as_view(), name="create"),
    re_path(
        "^(?P<pk>[0-9]+)/edit/$",
        views.ServiceUpdate.as_view(),
        name="edit",
    ),
    re_path(
        "^(?P<pk>[0-9]+)/delete/$",
        views.ServiceDelete.as_view(),
        name="delete",
    ),
]

servicegroup_urls = [
    re_path("^create/$", views.ServiceGroupCreate.as_view(), name="create"),
    re_path(
        "^(?P<pk>[0-9]+)/edit/$",
        views.ServiceGroupUpdate.as_view(),
        name="edit",
    ),
    re_path(
        "^(?P<pk>[0-9]+)/delete/$",
        views.ServiceGroupDelete.as_view(),
        name="delete",
    ),
]

maintenance_urls = [
    re_path("^create/$", views.MaintenanceCreate.as_view(), name="create"),
    re_path(
        "^(?P<pk>[0-9]+)/edit/$",
        views.MaintenanceUpdate.as_view(),
        name="edit",
    ),
    re_path(
        "^(?P<pk>[0-9]+)/delete/$",
        views.MaintenanceDelete.as_view(),
        name="delete",
    ),
]

incident_urls = [
    re_path("^archive/$", views.incident_archive_index, name="archive-index"),
    re_path(
        "^archive/(?P<year>[0-9]{4})/(?P<month>[0-9]{1,2})/$",
        views.IncidentMonthArchiveView.as_view(),
        name="archive-month",
    ),
    re_path("^create/$", views.incident_create, name="create"),
    re_path("^(?P<pk>[0-9]+)/edit/$", views.incident_edit, name="edit"),
    re_path(
        "^(?P<pk>[0-9]+)/delete/$",
        views.IncidentDeleteView.as_view(),
        name="delete",
    ),
]

urlpatterns = [
    re_path(r"^$", views.index, name="index"),
]

urlpatterns += [
    re_path(
        r"^login/$",
        auth_views.LoginView.as_view(
            template_name="statusboard/login.html",
        ),
        name="login",
    ),
    re_path(
        r"^logout/$",
        auth_views.LogoutView.as_view(
            template_name="statusboard/login.html",
        ),
        name="logout",
    ),
]

urlpatterns += [
    re_path(r"^service/", include((service_urls, "service"))),
    re_path(r"^servicegroup/", include((servicegroup_urls, "servicegroup"))),
    re_path(r"^incident/", include((incident_urls, "incident"))),
    re_path(r"^maintenance/", include((maintenance_urls, "maintenance"))),
    re_path(r"^api/v0.1/", include((router.urls, "api"), namespace="v0.1")),
]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from datetime import datetime, timedelta

from django.test import TestCase
from django.test import Client
from django.contrib.auth.models import User, Permission
from django.utils import timezone

from rest_framework.test import APIClient

from statusboard.models import Service
from statusboard.models import ServiceGroup
from statusboard.models import Incident
from statusboard.models import IncidentUpdate
from statusboard.models import Maintenance


class TestApiPermission(TestCase):
    def setUp(self):
        self.anon_client = APIClient()

        createuser = User.objects.create_user(username="create")
        createuser.user_permissions.add(
            Permission.objects.get(codename="add_servicegroup")
        )
        createuser.save()
        self.create_client = APIClient()
        self.create_client.force_authenticate(createuser)

        deleteuser = User.objects.create_user(username="delete")
        deleteuser.user_permissions.add(
            Permission.objects.get(codename="delete_servicegroup")
        )
        deleteuser.save()
        self.delete_client = APIClient()
        self.delete_client.force_authenticate(deleteuser)

        edituser = User.objects.create_user(username="edit")
        edituser.user_permissions.add(Permission.objects.get(
            codename="change_servicegroup")
        )
        edituser.save()
        self.edit_client = APIClient()
        self.edit_client.force_authenticate(edituser)

    def test_servicegroup(self):
        """Test service group API permissions"""
        response = self.anon_client.get("/statusboard/api/v0.1/servicegroup/")
        self.assertEqual(response.status_code, 200)
        response = self.anon_client.post("/statusboard/api/v0.1/servicegroup/")
        self.assertEqual(response.status_code, 403)

        response = self.create_client.post(
            "/statusboard/api/v0.1/servicegroup/", {
                "name": "test",
            }
        )
        self.assertEqual(response.status_code, 201)
        self.assertTrue(ServiceGroup.objects.get(name="test") is not None)

        response = self.delete_client.patch(
            "/statusboard/api/v0.1/servicegroup/1/", {
                "name": "t",
            }
        )
        self.assertEqual(response.status_code, 403)
        self.assertTrue(ServiceGroup.objects.get(name="test") is not None)

        response = self.edit_client.patch(
            "/statusboard/api/v0.1/servicegroup/1/", {
                "name": "t",
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertTrue(ServiceGroup.objects.get(name="t") is not None)

        response = self.create_client.delete(
            "/statusboard/api/v0.1/servicegroup/1/"
        )
        self.assertEqual(response.status_code, 403)
        self.assertEqual(ServiceGroup.objects.filter(pk=1).count(), 1)

        response = self.delete_client.delete(
            "/statusboard/api/v0.1/servicegroup/1/"
        )
        self.assertEqual(response.status_code, 204)
        self.assertEqual(ServiceGroup.objects.filter(pk=1).count(), 0)


class TestTemplate(TestCase):
    def setUp(self):
        admin = User.objects.create_superuser(username="admin",
                                              password="admin",
                                              email="admin@admin")
        admin.save()

    def test_service_create(self):
        client = Client()
        client.login(username="admin", password="admin")
        response = client.get('/statusboard/service/create/')
        self.assertEqual(response.status_code, 200)
        templates = [t.name for t in response.templates]
        self.assertTrue('statusboard/base.html' in templates)
        self.assertTrue('statusboard/service/create.html' in templates)
        self.assertTrue('statusboard/service/form.html' in templates)

    def test_service_delete(self):
        client = Client()
        client.login(username="admin", password="admin")
        Service.objects.create(pk=1, name="test", status=0)
        response = client.get('/statusboard/service/1/delete/')
        self.assertEqual(response.status_code, 200)
        templates = [t.name for t in response.templates]
        self.assertTrue('statusboard/base.html' in templates)
        self.assertTrue('statusboard/service/confirm_delete.html' in templates)


    def test_maintenance_create(self):
        client = Client()
        client.login(username="admin", password="admin")
        response = client.get('/statusboard/maintenance/create/')
        self.assertEqual(response.status_code, 200)
        templates = [t.name for t in response.templates]
        self.assertTrue('statusboard/base.html' in templates)
        self.assertTrue('statusboard/maintenance/create.html' in templates)
        self.assertTrue('statusboard/maintenance/form.html' in templates)

    def test_maintenance_delete(self):
        client = Client()
        client.login(username="admin", password="admin")
        Maintenance.objects.create(pk=1, scheduled=timezone.now(), name="test", description="test")
        response = client.get('/statusboard/maintenance/1/delete/')
        self.assertEqual(response.status_code, 200)
        templates = [t.name for t in response.templates]
        self.assertTrue('statusboard/base.html' in templates)
        self.assertTrue('statusboard/maintenance/confirm_delete.html' in templates)

    def test_index_maintenance(self):
        client = Client()
        Maintenance.objects.create(
            pk=1,
            scheduled=timezone.now() + timedelta(hours=1),
            name="maintenance-name-1",
            description="maintenance-description-1",
        )
        Maintenance.objects.create(
            pk=2,
            scheduled=timezone.now() - timedelta(hours=1),
            name="maintenance-name-2",
            description="maintenance-description-2",
        )
        response = client.get('/statusboard/')
        self.assertContains(response, text='maintenance-name-1')
        self.assertContains(response, text='maintenance-description-1')
        self.assertNotContains(response, text='maintenance-name-2')
        self.assertNotContains(response, text='maintenance-description-2')

    def test_index_incident(self):
        client = Client()
        Incident.objects.create(pk=1, name="incident-name-1", occurred=timezone.now(), closed=False)
        response = client.get('/statusboard/')
        self.assertContains(response, text="incident-name-1")
        templates = [t.name for t in response.templates]
        self.assertTrue("statusboard/incident/list_snippet.html" in templates)

    def test_index_open_past_incident(self):
        client = Client()
        Incident.objects.create(pk=1, name="incident-name-1", occurred=timezone.now()-timedelta(days=1), closed=False)
        Incident.objects.create(pk=2, name="incident-name-2", occurred=timezone.now()-timedelta(days=1), closed=True)
        with self.settings(STATUSBOARD={
            'OPEN_INCIDENT_IN_INDEX': True,
            'INCIDENT_DAYS_IN_INDEX': 3,
        }):
            response = client.get('/statusboard/')
            self.assertContains(response, text="incident-name-1")
            self.assertContains(response, text="incident-name-2")

        with self.settings(STATUSBOARD={
            'INCIDENT_DAYS_IN_INDEX': 1,
            'OPEN_INCIDENT_IN_INDEX': False,
        }):
            response = client.get('/statusboard/')
            self.assertNotContains(response, text="incident-name-1")
            self.assertNotContains(response, text="incident-name-2")

        with self.settings(STATUSBOARD={
            'INCIDENT_DAYS_IN_INDEX': 2,
        }):
            response = client.get('/statusboard/')
            self.assertContains(response, text="incident-name-1")
            self.assertContains(response, text="incident-name-2")

    def test_index_servicegroup(self):
        client = Client()
        servicegroup = ServiceGroup.objects.create(name="servicegroup-name")
        service = Service.objects.create(name="s1", description="s1", status=2)
        service.groups.add(servicegroup)
        response = client.get('/statusboard/')
        templates = [t.name for t in response.templates]
        self.assertTrue('statusboard/servicegroup/list_snippet.html' in templates)
        self.assertContains(response, text='servicegroup-name')

    def test_index_refresh(self):
        client = Client()

        # Default: no refresh
        response = client.get('/statusboard/')
        self.assertNotContains(response, text='<meta http-equiv="refresh"')

        with self.settings(STATUSBOARD={
            "AUTO_REFRESH_INDEX_SECONDS": 0
        }):
            response = client.get('/statusboard/')
            self.assertNotContains(response, text='<meta http-equiv="refresh"')

        with self.settings(STATUSBOARD={
            "AUTO_REFRESH_INDEX_SECONDS": 1
        }):
            response = client.get('/statusboard/')
            self.assertContains(response,
                                text='<meta http-equiv="refresh" content="1">')

    def test_index_favicon(self):
        from statusboard.settings import statusconf

        client = Client()
        # Test default favicons
        response = client.get('/statusboard/')
        self.assertContains(response, status_code=200,
                            text=statusconf.FAVICON_DEFAULT)

        s1 = Service.objects.create(name="s1", description="test", status=0)
        for status in (0, 1, 2, 3):
            s1.status = status
            s1.save()
            response = client.get('/statusboard/')
            self.assertContains(response, status_code=200,
                                text=statusconf.FAVICON_INDEX_DICT[status])

        # This service is always operational so it doesn't affect the icon
        # image
        Service.objects.create(name="s2", description="test", status=0)

        # Test custom favicons
        with self.settings(STATUSBOARD={
            'FAVICON_DEFAULT': 'default',
            'FAVICON_INDEX_DICT': {
                0: '0',
                1: '1',
                2: '2',
                3: '3',
            },
        }):
            for status in (0, 1, 2, 3):
                s1.status = status
                s1.save()
                response = client.get('/statusboard/')
                self.assertContains(response, status_code=200,
                                    text=statusconf.FAVICON_INDEX_DICT[status])

        # Test custom favicons (default only)
        with self.settings(STATUSBOARD={
            'FAVICON_DEFAULT': 'default',
            'FAVICON_INDEX_DICT': {}
        }):
            for status in (0, 1, 2, 3):
                s1.status = status
                s1.save()
                response = client.get('/statusboard/')
                self.assertContains(response, status_code=200,
                                    text=statusconf.FAVICON_DEFAULT)


class TestIncidentCreate(TestCase):
    def setUp(self):
        admin = User.objects.create_superuser(username="admin",
                                              password="admin",
                                              email="admin@admin")
        admin.save()

        Service.objects.create(name="s1", description="s1", status=2)
        Service.objects.create(name="s2", description="s2", status=2)

    def test_create(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/create/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [1, 2],
            'service_status': 0,
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        incident = Incident.objects.get(pk=1)
        self.assertEqual(incident.services.count(), 2)
        for s in incident.services.all():
            self.assertEqual(s.status, 0)

    def test_create_without_services(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/create/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [],
            'service_status': 0,
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        incident = Incident.objects.get(pk=1)
        self.assertEqual(incident.services.count(), 0)

    def test_create_with_empty_service_status(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/create/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [1, 2],
            'service_status': '',
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        incident = Incident.objects.get(pk=1)
        self.assertEqual(incident.services.count(), 2)
        for s in incident.services.all():
            self.assertEqual(s.status, 2)


class TestIncidentEdit(TestCase):
    def setUp(self):
        admin = User.objects.create_superuser(username="admin",
                                              password="admin",
                                              email="admin@admin")
        admin.save()

        s1 = Service.objects.create(name="s1", description="s1", status=2)
        s2 = Service.objects.create(name="s2", description="s2", status=2)
        i = Incident(name="incident")
        i.full_clean()
        i.save()
        i.services.add(s1)
        i.services.add(s2)
        i.full_clean()
        i.save()

    def test_edit(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/1/edit/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [1, 2],
            'service_status': 0,
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        incident = Incident.objects.get(pk=1)
        self.assertEqual(incident.services.count(), 2)
        for s in incident.services.all():
            self.assertEqual(s.status, 0)

    def test_edit_with_empty_service_status(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/1/edit/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [1, 2],
            'service_status': '',
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        incident = Incident.objects.get(pk=1)
        self.assertEqual(incident.services.count(), 2)
        for s in incident.services.all():
            self.assertEqual(s.status, 2)

    def test_edit_remove_service(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/1/edit/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': 1,
            'service_status': 0,
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        i = Incident.objects.get(pk=1)
        self.assertEqual(i.services.count(), 1)

    def test_valid_status(self):
        client = Client()
        client.login(username="admin", password="admin")
        client.post('/statusboard/incident/1/edit/', {
            'name': 'incident',
            'occurred': '2010-01-01 00:00:00',
            'services': [1, 2],
            'service_status': 33,
            'updates-INITIAL_FORMS': 0,
            'updates-TOTAL_FORMS': 0,
            'updates-MAX_NUM_FORMS': 0,
            'updates-MIN_NUM_FORMS': 0,
        })
        i = Incident.objects.get(pk=1)
        s = i.services.first()
        self.assertEqual(s.status, 2)


class TestServiceGroup(TestCase):
    def test_priority_order(self):
        s0 = ServiceGroup.objects.create(name="s0", priority=1)
        s1 = ServiceGroup.objects.create(name="s1", priority=0)
        self.assertEqual(ServiceGroup.objects.all()[0], s0)
        self.assertEqual(ServiceGroup.objects.all()[1], s1)
        self.assertEqual(ServiceGroup.objects.priority_sorted()[0], s0)
        self.assertEqual(ServiceGroup.objects.priority_sorted()[1], s1)
        # When two groups have the same priority, order by name
        s2 = ServiceGroup.objects.create(name="s2", priority=0)
        self.assertEqual(ServiceGroup.objects.priority_sorted()[0], s0)
        self.assertEqual(ServiceGroup.objects.priority_sorted()[1], s1)
        self.assertEqual(ServiceGroup.objects.priority_sorted()[2], s2)
        # Filter queryset object
        self.assertEqual(
            ServiceGroup.objects.filter(name="s1").priority_sorted()[0], s1
        )

    def test_worst_service(self):
        g = ServiceGroup.objects.create(name="test", collapse=0)
        self.assertRaises(Service.DoesNotExist, g.worst_service)
        s = Service.objects.create(name="s0", description="test", status=0)
        g.services.add(s)
        g.save()
        self.assertEqual(g.worst_service(), s)
        s = Service.objects.create(name="s1", description="test", status=1)
        g.services.add(s)
        g.save()
        self.assertEqual(g.worst_service(), s)

    def test_is_empty_group(self):
        g = ServiceGroup.objects.create(name="test", collapse=0)
        self.assertTrue(g.is_empty_group())
        s = Service.objects.create(name="s1", description="test", status=1)
        g.services.add(s)
        g.save()
        self.assertFalse(g.is_empty_group())


class TestIncidentManager(TestCase):
    def setUp(self):
        dt = timezone.now()
        days = 30
        i = Incident.objects.create(name="a", occurred=dt)
        IncidentUpdate.objects.create(incident=i, status=0, description="test")
        IncidentUpdate.objects.create(incident=i, status=3, description="test")
        i = Incident.objects.create(name="b",
                                    occurred=dt-timezone.timedelta(days=days))
        IncidentUpdate.objects.create(incident=i, status=0, description="test")
        self.days = days

    def test_occurred_in_last_n_days(self):
        self.assertEqual(
            Incident.objects.occurred_in_last_n_days(self.days-1).count(), 1
        )
        self.assertEqual(
            Incident.objects.occurred_in_last_n_days(self.days).count(), 1
        )

    def test_last_occurred(self):
        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": self.days+2,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 2)

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": self.days+1,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 2)

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": self.days,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 1)

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": self.days-1,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 1)

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": 1,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 1)

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": 0,
        }):
            self.assertEqual(Incident.objects.last_occurred().count(), 0)

    def test_in_index(self):
        self.assertEqual(Incident.objects.in_index().count(), 2)

        with self.settings(STATUSBOARD={
            "OPEN_INCIDENT_IN_INDEX": True,
            "INCIDENT_DAYS_IN_INDEX": self.days+1,
        }):
            self.assertEqual(Incident.objects.in_index().count(), 2)

        with self.settings(STATUSBOARD={
            "OPEN_INCIDENT_IN_INDEX": True,
            "INCIDENT_DAYS_IN_INDEX": 0,
        }):
            self.assertEqual(Incident.objects.in_index().count(), 1)

        with self.settings(STATUSBOARD={
            "OPEN_INCIDENT_IN_INDEX": False,
            "INCIDENT_DAYS_IN_INDEX": 0,
        }):
            self.assertEqual(Incident.objects.in_index().count(), 0)


class TestIncident(TestCase):
    def test_closed(self):
        i = Incident.objects.create(name="incident")
        self.assertFalse(i.closed)
        IncidentUpdate.objects.create(incident=i, status=0, description="test")
        self.assertFalse(i.closed)
        IncidentUpdate.objects.create(incident=i, status=3, description="test")
        self.assertTrue(i.closed)
        IncidentUpdate.objects.create(incident=i, status=0, description="test")
        self.assertFalse(i.closed)


class TestTemplateTags(TestCase):
    def test_servicegroup_collapse(self):
        g = ServiceGroup.objects.create(name="test", collapse=0)
        self.assertFalse(g.collapsed())

        g.collapse = 1
        g.save()
        self.assertTrue(g.collapsed())

        g.collapse = 2
        g.save()
        self.assertTrue(g.collapsed())

        s = Service.objects.create(name="service", description="test",
                                   status=0)
        s.groups.add(g)
        s.save()
        g.collapse = 2
        g.save()
        self.assertTrue(g.collapsed())

        s.status = 1
        s.save()
        self.assertFalse(g.collapsed())


class TestService(TestCase):
    def test_worst_status(self):
        # ServiceManager
        self.assertEqual(Service.objects.worst_status(), None)
        # ServiceQuerySet
        self.assertEqual(Service.objects.all().worst_status(), None)

    def test_previous_status(self):
        from django.db.models.signals import post_save

        def check_different(sender, instance, **kwargs):
            check_different.is_different = (
                instance._status != instance.status
            )

        check_different.is_different = None

        post_save.connect(check_different)

        s = Service(name="service", description="test", status=0)
        s.save()
        self.assertFalse(check_different.is_different)
        s.status = 0
        s.save()
        self.assertFalse(check_different.is_different)
        s.status = 1
        s.save()
        self.assertIsNotNone(check_different.is_different)
        self.assertTrue(check_different.is_different)

        post_save.disconnect(check_different)


class TestSettings(TestCase):
    def test_default(self):
        from statusboard.settings import statusconf

        self.assertIsNotNone(getattr(statusconf, "INCIDENT_DAYS_IN_INDEX"))
        self.assertTrue(hasattr(statusconf, "INCIDENT_DAYS_IN_INDEX"))

    def test_modified(self):
        from django.conf import settings
        from statusboard.settings import statusconf

        with self.settings(STATUSBOARD={
            "INCIDENT_DAYS_IN_INDEX": 30,
        }):
            self.assertEqual(statusconf.INCIDENT_DAYS_IN_INDEX, 30)
            self.assertEqual(settings.STATUSBOARD["INCIDENT_DAYS_IN_INDEX"],
                             statusconf.INCIDENT_DAYS_IN_INDEX)

    def test_missing_default(self):
        from statusboard.settings import statusconf

        with self.assertRaises(RuntimeError):
            statusconf.THIS_IS_A_MISSING_APP_SETTING


class TestMaintenanceEdit(TestCase):
    def setUp(self):
        admin = User.objects.create_superuser(username="admin",
                                              password="admin",
                                              email="admin@admin")
        admin.save()

        m = Maintenance(scheduled=timezone.now(),
                        name="test",
                        description="test")
        m.full_clean()
        m.save()

    def test_edit(self):
        client = Client()
        client.login(username="admin", password="admin")
        dt = timezone.now()
        client.post('/statusboard/maintenance/1/edit/', {
            'scheduled': dt,
            'name': 'modified name',
            'description': 'modified description',
        })
        m = Maintenance.objects.get(pk=1)
        self.assertEqual(m.scheduled, dt)
        self.assertEqual(m.name, "modified name")
        self.assertEqual(m.description, "modified description")


class TestReverseAndResolve(TestCase):
    def test_maintenance(self):
        from django.urls import reverse, resolve

        self.assertEqual(
            reverse('statusboard:maintenance:create'),
            '/statusboard/maintenance/create/',
        )
        resolver = resolve('/statusboard/maintenance/create/')
        self.assertEqual(
            resolver.namespaces,
            ['statusboard', 'maintenance'],
        )
        self.assertEqual(
            resolver.view_name,
            'statusboard:maintenance:create',
        )


class TestPermissionRequiredView(TestCase):
    def test_permission_required(self):
        with self.settings(LOGIN_URL='/login'):
            client = Client()
            response = client.get('/statusboard/service/create/')
            self.assertRedirects(response,
                                 '/login?next=/statusboard/service/create/',
                                 fetch_redirect_response=False)


class TestChangePermission(TestCase):
    def setUp(self):
        edituser = User.objects.create_user(username="edit", password="edit")
        edituser.user_permissions.add(
            Permission.objects.get(codename="change_service")
        )
        edituser.save()
        self.client = Client()
        self.client.login(username='edit', password='edit')

        self.servicegroup = ServiceGroup.objects.create(name="g1")
        self.service = Service.objects.create(name="s1", description="s1",
                                              status=2)

    def test_change_service(self):
        self.client.post(
            '/statusboard/service/{}/edit/'.format(self.service.pk), {
                'name': 's1-changed',
                'description': self.service.description,
                'href': self.service.href,
                'status': self.service.status,
                'priority': self.service.priority,
                'groups': [self.servicegroup.pk],
            }
        )
        self.assertEqual(Service.objects.get(pk=self.service.pk).name,
                         's1-changed')


class TestArchiveIncident(TestCase):
    def test_archive_index_empty(self):
        from django.urls import reverse
        client = Client()
        resp = client.get(reverse('statusboard:incident:archive-index'))
        self.assertTrue('statusboard/incident/archive_month_empty.html' in [
            t.name for t in resp.templates
        ])

    def test_archive_index_nonempty(self):
        from django.urls import reverse
        i = Incident.objects.create(name="incident")
        client = Client()
        resp = client.get(reverse('statusboard:incident:archive-index'))
        self.assertRedirects(
            resp,
            reverse('statusboard:incident:archive-month', args=[
                i.created.year, i.created.month
            ])
        )



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.templatetags.static import static


"""
The settings for the app can be set in the project settings, e.g.:

    STATUSBOARD = {
        "INCIDENT_DAYS_IN_INDEX": 20,
    }
"""


DEFAULTS = {
    # Number of incidents in index (1 = today, 0 = deactivate)
    "INCIDENT_DAYS_IN_INDEX": 7,
    # Show or not all the open incidents in index
    "OPEN_INCIDENT_IN_INDEX": True,
    # Automatic refresth the home page in seconds (disable with 0)
    "AUTO_REFRESH_INDEX_SECONDS": 0,
    # Default favicon
    "FAVICON_DEFAULT": static("statusboard/favicons/statusboard-icon-default.ico"),
    # Favicons for index
    "FAVICON_INDEX_DICT": {
        0: static("statusboard/favicons/statusboard-icon-operational.ico"),
        1: static("statusboard/favicons/statusboard-icon-performance.ico"),
        2: static("statusboard/favicons/statusboard-icon-partial.ico"),
        3: static("statusboard/favicons/statusboard-icon-major.ico"),
    },
}


class Settings(object):
    """Settings for internal use."""

    def __getattr__(self, name):
        if name not in DEFAULTS:
            raise RuntimeError(("{} is not a valid statusboard " "setting").format(name))
        s = getattr(settings, "STATUSBOARD", {})
        return s.get(name, DEFAULTS[name])


statusconf = Settings()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 15:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0018_merge_20170223_1429'),
    ]

    operations = [
        migrations.AlterField(
            model_name='service',
            name='priority',
            field=models.PositiveIntegerField(default=0, verbose_name='priority'),
        ),
        migrations.AlterField(
            model_name='servicegroup',
            name='priority',
            field=models.PositiveIntegerField(default=0, verbose_name='priority'),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.contrib import admin

from .models import (
    Service,
    ServiceGroup,
    Incident,
    IncidentUpdate,
    Maintenance,
)


class IncidentUpdateInline(admin.TabularInline):
    model = IncidentUpdate
    ordering = ["created"]


class IncidentAdmin(admin.ModelAdmin):
    inlines = [IncidentUpdateInline]


admin.site.register(Service)
admin.site.register(ServiceGroup)
admin.site.register(Incident, IncidentAdmin)
admin.site.register(Maintenance)



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import os


USE_TZ = True
TIME_ZONE = "Europe/Rome"

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'rest_framework',
    'statusboard',
]

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'debug': True,
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ":memory:"
    }
}

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
]

STATIC_URL = '/static/'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-16 10:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0007_auto_20170213_1114'),
    ]

    operations = [
        migrations.AddField(
            model_name='servicegroup',
            name='collapse',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-01-25 06:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0003_auto_20170125_0604'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='occurred',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-01-25 06:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0002_incident'),
    ]

    operations = [
        migrations.CreateModel(
            name='IncidentUpdate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('status', models.IntegerField(choices=[(0, 'Investigating'), (1, 'Identified'), (2, 'Watching'), (3, 'Fixed')])),
                ('description', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='incident',
            name='description',
        ),
        migrations.RemoveField(
            model_name='incident',
            name='occurred',
        ),
        migrations.RemoveField(
            model_name='incident',
            name='status',
        ),
        migrations.AddField(
            model_name='incidentupdate',
            name='incident',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='updates', related_query_name='update', to='statusboard.Incident'),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from rest_framework import serializers

from .models import (
    Service,
    ServiceGroup,
    Incident,
    IncidentUpdate,
    Maintenance,
)


class ServiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Service
        fields = (
            "id",
            "name",
            "description",
            "href",
            "status",
            "groups",
            "priority",
            "created",
            "modified",
        )


class ServiceGroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = ServiceGroup
        fields = (
            "id",
            "name",
            "collapse",
            "priority",
            "services",
            "created",
            "modified",
        )


class IncidentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Incident
        fields = (
            "id",
            "name",
            "services",
            "occurred",
            "created",
            "modified",
            "updates",
            "closed",
        )
        read_only_fields = ("closed",)


class IncidentUpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = IncidentUpdate
        fields = ("id", "status", "description", "created", "modified")


class MaintenanceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Maintenance
        fields = (
            "id",
            "name",
            "description",
            "created",
            "modified",
            "scheduled",
        )



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.dispatch import receiver

from model_utils.models import TimeStampedModel

from .settings import statusconf


SERVICE_STATUSES = (
    (0, _("Operational")),
    (1, _("Performance issues")),
    (2, _("Partial outage")),
    (3, _("Major outage")),
)


class ServiceQuerySet(models.QuerySet):
    def worst_status(self):
        """Return worst status in queryset."""
        return self.aggregate(models.Max("status"))["status__max"]

    def worst_service(self):
        return self.latest("status")

    def uncategorized(self):
        return self.annotate(group_count=models.Count("groups")).filter(group_count=0)

    def priority_sorted(self):
        return self.order_by("-priority", "name")


class ServiceManager(models.Manager):
    def get_queryset(self):
        return ServiceQuerySet(self.model, using=self._db)

    def worst_status(self):
        """Return worst status in queryset."""
        return self.get_queryset().worst_status()

    def uncategorized(self):
        return self.get_queryset().uncategorized()

    def priority_sorted(self):
        return self.get_queryset().priority_sorted()


class Service(TimeStampedModel):
    name = models.CharField(max_length=255, unique=True, verbose_name=_("name"))
    description = models.TextField(blank=True, verbose_name=_("description"))
    href = models.URLField(blank=True)
    status = models.IntegerField(choices=SERVICE_STATUSES, verbose_name=_("status"))
    priority = models.PositiveIntegerField(default=0, verbose_name=_("priority"))
    groups = models.ManyToManyField(
        "ServiceGroup",
        blank=True,
        related_name="services",
        related_query_name="service",
        verbose_name=_("groups"),
    )
    objects = ServiceManager()

    def __str__(self):
        return self.name

    def __init__(self, *args, **kwargs):
        super(Service, self).__init__(*args, **kwargs)
        self._status = self.status

    def save(self, *args, **kwargs):
        # When the instance is saved, the original status is updated
        super(Service, self).save(*args, **kwargs)
        self._status = self.status

    class Meta:
        verbose_name = _("service")
        verbose_name_plural = _("services")


SERVICEGROUP_COLLAPSE_OPTIONS = (
    (0, _("Never collapse")),
    (1, _("Always collapse")),
    (2, _("Collapse when all the services are operational")),
)


class ServiceGroupQuerySet(models.QuerySet):
    def priority_sorted(self):
        return self.order_by("-priority", "name")


class ServiceGroupManager(models.Manager):
    def get_queryset(self):
        return ServiceGroupQuerySet(self.model, using=self._db)

    def priority_sorted(self):
        return self.get_queryset().priority_sorted()


class ServiceGroup(TimeStampedModel):
    name = models.CharField(max_length=255, unique=True, verbose_name=_("name"))
    priority = models.PositiveIntegerField(default=0, verbose_name=_("priority"))
    collapse = models.IntegerField(choices=SERVICEGROUP_COLLAPSE_OPTIONS, default=0)
    objects = ServiceGroupManager()

    def worst_service(self):
        return self.services.all().latest("status")

    def collapsed(self):
        """Check if the service group should collapse or not.

        Return true if a group doesn't have any service."""
        if self.collapse == 0:
            return False
        elif self.collapse == 1:
            return True
        else:
            return not self.services.exclude(status=0).exists()

    def is_empty_group(self):
        return not self.services.all().exists()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("service group")
        verbose_name_plural = _("service groups")


INCIDENT_STATUSES = (
    (0, _("Investigating")),
    (1, _("Identified")),
    (2, _("Watching")),
    (3, _("Fixed")),
)


class IncidentQuerySet(models.QuerySet):
    def occurred_in_last_n_days_q(self, days):
        """Q obects for the incidents occurred in last N days (1 = today)."""
        threshold = timezone.now() - timezone.timedelta(days=days - 1)
        threshold = threshold.replace(hour=0, minute=0, second=0, microsecond=0)
        return models.Q(occurred__gte=threshold)

    def last_occurred_q(self):
        return self.occurred_in_last_n_days_q(statusconf.INCIDENT_DAYS_IN_INDEX)

    def occurred_in_last_n_days(self, days=7):
        """Return incidents occurred in last N days (1 = today)."""
        return self.filter(self.occurred_in_last_n_days_q(days))

    def last_occurred(self):
        """Return incidents occurred in last days. The number of days is
        defined in STATUSBOARD["INCIDENT_DAYS_IN_INDEX"]."""
        return self.filter(self.last_occurred_q())

    def in_index(self):
        q = self.last_occurred_q()
        if statusconf.OPEN_INCIDENT_IN_INDEX:
            q = q | models.Q(closed=False)

        return self.filter(q)


class IncidentManager(models.Manager):
    def get_queryset(self):
        return IncidentQuerySet(self.model, using=self._db)

    def occurred_in_last_n_days(self, days=7):
        """Return incidents occurred in last N days (1 = today)."""
        return self.get_queryset().occurred_in_last_n_days(days=days)

    def last_occurred(self):
        """Return incidents occurred in last days. The number of days is
        defined in STATUSBOARD["INCIDENT_DAYS_IN_INDEX"]."""
        return self.get_queryset().last_occurred()

    def in_index(self):
        return self.get_queryset().in_index()


class Incident(TimeStampedModel):
    name = models.CharField(max_length=255, verbose_name=_("name"))
    services = models.ManyToManyField(
        "Service",
        blank=True,
        related_name="incidents",
        related_query_name="incident",
        verbose_name=_("services"),
    )
    occurred = models.DateTimeField(default=timezone.now, verbose_name=_("occurred"))
    closed = models.BooleanField(default=False)
    objects = IncidentManager()

    def updates_by_ctime(self):
        return self.updates.order_by("-created")

    def should_be_closed(self):
        try:
            return self.updates.latest("created").status == 3
        except IncidentUpdate.DoesNotExist:
            return False

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("incident")
        verbose_name_plural = _("incidents")


class IncidentUpdate(TimeStampedModel):
    incident = models.ForeignKey(
        Incident,
        related_name="updates",
        on_delete=models.deletion.CASCADE,
        related_query_name="update",
        verbose_name=_("incident"),
    )
    status = models.IntegerField(choices=INCIDENT_STATUSES, verbose_name=_("status"))
    description = models.TextField(verbose_name=_("description"))

    def __str__(self):
        return "Update {} {}".format(self.incident.name, self.modified)

    class Meta:
        verbose_name = _("incident update")
        verbose_name_plural = _("incident updates")


class Maintenance(TimeStampedModel):
    scheduled = models.DateTimeField(verbose_name=_("scheduled start"))
    name = models.CharField(max_length=255, verbose_name=_("name"))
    description = models.TextField(verbose_name=_("description"))

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("maintenance")
        verbose_name_plural = _("maintenances")


@receiver(models.signals.post_save, sender=IncidentUpdate)
@receiver(models.signals.post_delete, sender=IncidentUpdate)
def update_incident_close_field(sender, instance, **kwargs):
    try:
        instance.incident.closed = instance.incident.should_be_closed()
        instance.incident.save()
    except IncidentUpdate.DoesNotExist:
        pass



# Generated by Django 3.0.10 on 2020-12-04 08:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0020_auto_20170223_1651'),
    ]

    operations = [
        migrations.AlterField(
            model_name='service',
            name='groups',
            field=models.ManyToManyField(blank=True, related_name='services', related_query_name='service', to='statusboard.ServiceGroup', verbose_name='groups'),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals
from __future__ import absolute_import

from django import template
from django import forms
from django.utils.translation import gettext as _


register = template.Library()


@register.filter
def service_status_summary_text(value, arg):
    return {
        "0": _("All systems are operational"),
        "1": _("Some systems are experiencing performance issues"),
        "2": _("Some systems are experiencing partial outages"),
        "3": _("Some systems are experiencing major outages"),
    }.get(str(value), arg)


@register.filter
def service_status_class(value, arg):
    return {
        "0": "success",
        "1": "info",
        "2": "warning",
        "3": "danger",
    }.get(str(value), arg)


@register.filter
def is_checkbox(field):
    return isinstance(field.field.widget, forms.CheckboxInput)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 21:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0015_merge_20170222_2058'),
    ]

    operations = [
        migrations.AddField(
            model_name='service',
            name='position',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 06:34
from __future__ import unicode_literals

from django.db import migrations


def close_previously_fixed_incidents_forward(apps, schema_editor):
    """Close incidents that were fixed before introducing the "closed"
    attribute."""
    Incident = apps.get_model("statusboard", "incident")
    db_alias = schema_editor.connection.alias
    for i in Incident.objects.using(db_alias).order_by("-created").all():
        if i.updates.exists():
            i.closed = i.updates.latest("created").status == 3
            i.save()


def close_previously_fixed_incidents_backward(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0014_incident_closed'),
    ]

    operations = [
        migrations.RunPython(close_previously_fixed_incidents_forward,
                             close_previously_fixed_incidents_backward)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 14:29
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0016_auto_20170223_1005'),
        ('statusboard', '0017_auto_20170223_1428'),
    ]

    operations = [
    ]



# Generated by Django 3.0.10 on 2020-12-04 08:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0021_auto_20201204_0846'),
    ]

    operations = [
        migrations.AlterField(
            model_name='maintenance',
            name='scheduled',
            field=models.DateTimeField(verbose_name='scheduled start'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 16:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0019_auto_20170223_1503'),
    ]

    operations = [
        migrations.AlterField(
            model_name='service',
            name='description',
            field=models.TextField(blank=True, verbose_name='description'),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.shortcuts import render, redirect
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy, reverse
from django.views.generic.dates import MonthArchiveView
from django.views.generic.edit import CreateView
from django.views.generic.edit import UpdateView
from django.views.generic.edit import DeleteView
from django.utils import timezone
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.mixins import PermissionRequiredMixin

from rest_framework import viewsets
from rest_framework import permissions
from rest_framework import versioning

from .models import (
    Service,
    ServiceGroup,
    Incident,
    IncidentUpdate,
    Maintenance,
)
from .serializers import (
    ServiceSerializer,
    ServiceGroupSerializer,
    IncidentSerializer,
    IncidentUpdateSerializer,
    MaintenanceSerializer,
)
from .forms import (
    IncidentForm,
    IncidentUpdateFormSet,
    ServiceGroupForm,
    ServiceForm,
    MaintenanceForm,
)
from .settings import statusconf


def index(request):
    worst_status = Service.objects.worst_status()
    return render(
        request,
        "statusboard/index.html",
        {
            "servicegroups": ServiceGroup.objects.priority_sorted(),
            "uncategorized": Service.objects.uncategorized(),
            "worst_status": worst_status,
            "incidents": Incident.objects.in_index().order_by("-occurred"),
            "maintenances": Maintenance.objects.filter(scheduled__gt=timezone.now()).order_by("-scheduled"),
            "auto_refresh": statusconf.AUTO_REFRESH_INDEX_SECONDS,
            "favicon": statusconf.FAVICON_INDEX_DICT.get(
                worst_status,
                statusconf.FAVICON_DEFAULT,
            ),
        },
    )


class ServiceGroupCreate(PermissionRequiredMixin, CreateView):
    model = ServiceGroup
    form_class = ServiceGroupForm
    template_name = "statusboard/servicegroup/create.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.create_servicegroup"


class ServiceGroupUpdate(PermissionRequiredMixin, UpdateView):
    model = ServiceGroup
    form_class = ServiceGroupForm
    template_name = "statusboard/servicegroup/edit.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.change_servicegroup"


class ServiceGroupDelete(PermissionRequiredMixin, DeleteView):
    model = ServiceGroup
    template_name = "statusboard/servicegroup/confirm_delete.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.delete_servicegroup"


class ServiceCreate(PermissionRequiredMixin, CreateView):
    model = Service
    form_class = ServiceForm
    template_name = "statusboard/service/create.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.add_service"


class ServiceUpdate(PermissionRequiredMixin, UpdateView):
    model = Service
    form_class = ServiceForm
    template_name = "statusboard/service/edit.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.change_service"


class ServiceDelete(PermissionRequiredMixin, DeleteView):
    model = Service
    template_name = "statusboard/service/confirm_delete.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.delete_service"


class MaintenanceCreate(PermissionRequiredMixin, CreateView):
    model = Maintenance
    form_class = MaintenanceForm
    template_name = "statusboard/maintenance/create.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.create_maintenance"


class MaintenanceUpdate(PermissionRequiredMixin, UpdateView):
    model = Maintenance
    form_class = MaintenanceForm
    template_name = "statusboard/maintenance/edit.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.change_maintenance"


class MaintenanceDelete(PermissionRequiredMixin, DeleteView):
    model = Maintenance
    template_name = "statusboard/maintenance/confirm_delete.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.delete_maintenance"


@permission_required("statusboard.create_incident")
def incident_create(request):
    form = IncidentForm()
    incident_updates = IncidentUpdateFormSet()
    if request.POST:
        form = IncidentForm(request.POST)
        if form.is_valid():
            incident = form.save(commit=False)
            incident_updates = IncidentUpdateFormSet(request.POST, instance=incident)
            if incident_updates.is_valid():
                form.save()
                incident_updates.save()
                return HttpResponseRedirect(reverse("statusboard:index"))

    return render(
        request,
        "statusboard/incident/create.html",
        {
            "form": form,
            "incident_updates": incident_updates,
        },
    )


@permission_required("statusboard.change_incident")
def incident_edit(request, pk):
    incident = Incident.objects.get(pk=pk)
    form = IncidentForm(instance=incident)
    incident_updates = IncidentUpdateFormSet(instance=incident)

    if request.POST:
        form = IncidentForm(request.POST or None, instance=incident)
        if form.is_valid():
            incident = form.save(commit=False)
            incident_updates = IncidentUpdateFormSet(request.POST, request.FILES, instance=incident)
            if incident_updates.is_valid():
                form.save()
                incident_updates.save()
                return HttpResponseRedirect(reverse("statusboard:index"))

    return render(
        request,
        "statusboard/incident/edit.html",
        {
            "form": form,
            "incident_updates": incident_updates,
        },
    )


class IncidentDeleteView(PermissionRequiredMixin, DeleteView):
    model = Incident
    template_name = "statusboard/incident/confirm_delete.html"
    success_url = reverse_lazy("statusboard:index")
    permission_required = "statusboard.delete_incident"


def incident_archive_index(request):
    try:
        dt = Incident.objects.latest("occurred").occurred
        return redirect("statusboard:incident:archive-month", year=dt.year, month=dt.month)
    except Incident.DoesNotExist:
        return render(request, "statusboard/incident/archive_month_empty.html")


class IncidentMonthArchiveView(MonthArchiveView):
    queryset = Incident.objects.all().order_by("-occurred")
    date_field = "occurred"
    allow_future = False
    month_format = "%m"
    template_name = "statusboard/incident/archive_month.html"


class ServiceGroupViewSet(viewsets.ModelViewSet):
    serializer_class = ServiceGroupSerializer
    queryset = ServiceGroup.objects.all()
    permission_classes = [permissions.DjangoModelPermissionsOrAnonReadOnly]
    versioning_class = versioning.URLPathVersioning


class ServiceViewSet(viewsets.ModelViewSet):
    serializer_class = ServiceSerializer
    queryset = Service.objects.all()
    permission_classes = [permissions.DjangoModelPermissionsOrAnonReadOnly]
    versioning_class = versioning.URLPathVersioning


class IncidentViewSet(viewsets.ModelViewSet):
    serializer_class = IncidentSerializer
    queryset = Incident.objects.all()
    permission_classes = [permissions.DjangoModelPermissionsOrAnonReadOnly]
    versioning_class = versioning.URLPathVersioning


class IncidentUpdateViewSet(viewsets.ModelViewSet):
    serializer_class = IncidentUpdateSerializer
    queryset = IncidentUpdate.objects.all()
    permission_classes = [permissions.DjangoModelPermissionsOrAnonReadOnly]
    versioning_class = versioning.URLPathVersioning


class MaintenanceViewSet(viewsets.ModelViewSet):
    serializer_class = MaintenanceSerializer
    queryset = Maintenance.objects.all()
    permission_classes = [permissions.DjangoModelPermissionsOrAnonReadOnly]
    versioning_class = versioning.URLPathVersioning



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 19:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0013_auto_20170220_1444'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='closed',
            field=models.BooleanField(default=False),
        ),
    ]



# Copyright (C) 2017 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from __future__ import unicode_literals

from django.apps import AppConfig


class StatusboardConfig(AppConfig):
    name = "statusboard"
    default_auto_field = "django.db.models.AutoField"



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-09 08:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0005_merge'),
    ]

    operations = [
        migrations.CreateModel(
            name='Maintenance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('scheduled', models.DateTimeField()),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
import re

from setuptools import find_packages, setup


setup(
    name="django-statusboard",
    packages=find_packages(include=["statusboard", "statusboard.*"]),
    include_package_data=True,
    license='GPLv2+',
    description='Django app to show system status',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='http://github.com/edigiacomo/django-statusboard',
    author='Emanuele Di Giacomo',
    author_email="emanuele@digiacomo.cc",
    python_requires=">=3.6",
    install_requires=[
        'django>=2.2', 'djangorestframework!=3.15.0', 'django-model-utils', 'pytz'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 14:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0016_service_position'),
    ]

    operations = [
        migrations.RenameField(
            model_name='service',
            old_name='position',
            new_name='priority',
        ),
        migrations.RenameField(
            model_name='servicegroup',
            old_name='position',
            new_name='priority',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-17 11:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0010_auto_20170217_0835'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='incident',
            options={'verbose_name': 'incident', 'verbose_name_plural': 'incidents'},
        ),
        migrations.AlterModelOptions(
            name='incidentupdate',
            options={'verbose_name': 'incident update', 'verbose_name_plural': 'incident updates'},
        ),
        migrations.AlterModelOptions(
            name='maintenance',
            options={'verbose_name': 'maintenance', 'verbose_name_plural': 'maintenances'},
        ),
        migrations.AlterModelOptions(
            name='service',
            options={'verbose_name': 'service', 'verbose_name_plural': 'services'},
        ),
        migrations.AlterModelOptions(
            name='servicegroup',
            options={'verbose_name': 'service group', 'verbose_name_plural': 'service groups'},
        ),
        migrations.AlterField(
            model_name='incident',
            name='name',
            field=models.CharField(max_length=255, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='incident',
            name='occurred',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='occurred'),
        ),
        migrations.AlterField(
            model_name='incident',
            name='service',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='statusboard.Service', verbose_name='service'),
        ),
        migrations.AlterField(
            model_name='incidentupdate',
            name='description',
            field=models.TextField(verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='incidentupdate',
            name='incident',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='updates', related_query_name='update', to='statusboard.Incident', verbose_name='incident'),
        ),
        migrations.AlterField(
            model_name='incidentupdate',
            name='status',
            field=models.IntegerField(choices=[(0, 'Investigating'), (1, 'Identified'), (2, 'Watching'), (3, 'Fixed')], verbose_name='status'),
        ),
        migrations.AlterField(
            model_name='maintenance',
            name='description',
            field=models.TextField(verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='maintenance',
            name='name',
            field=models.CharField(max_length=255, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='maintenance',
            name='scheduled',
            field=models.DateTimeField(verbose_name='scheduled'),
        ),
        migrations.AlterField(
            model_name='service',
            name='description',
            field=models.TextField(verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='service',
            name='groups',
            field=models.ManyToManyField(related_name='services', related_query_name='service', to='statusboard.ServiceGroup', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='service',
            name='name',
            field=models.CharField(max_length=255, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='service',
            name='status',
            field=models.IntegerField(choices=[(0, 'Operational'), (1, 'Performance issues'), (2, 'Partial outage'), (3, 'Major outage')], verbose_name='status'),
        ),
        migrations.AlterField(
            model_name='servicegroup',
            name='collapse',
            field=models.IntegerField(choices=[(0, 'Never collapse'), (1, 'Always collapse'), (2, 'Collapse when a service is not operational')], default=0),
        ),
        migrations.AlterField(
            model_name='servicegroup',
            name='name',
            field=models.CharField(max_length=255, unique=True, verbose_name='name'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-17 07:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0008_servicegroup_collapse'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicegroup',
            name='collapse',
            field=models.IntegerField(choices=[(0, 'Never collapse'), (1, 'Always collapse'), (2, 'When service is not operational')], default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 10:05
from __future__ import unicode_literals

from django.db import migrations, models


def copy_service_to_services_forward(apps, schema_editor):
    """Copy the service from ForeignKeyField to ManyToManyField."""
    Incident = apps.get_model("statusboard", "incident")
    db_alias = schema_editor.connection.alias
    for i in Incident.objects.using(db_alias).filter(service__isnull=False).all():
        i.services.add(i.service)
        i.save()


def copy_service_to_services_backward(apps, schema_editor):
    """Copy the first service from ManyToManyField to ForeignKeyField."""
    Incident = apps.get_model("statusboard", "incident")
    db_alias = schema_editor.connection.alias
    for i in Incident.objects.using(db_alias).all():
        s = i.services.first()
        if s is not None:
            i.service = s
            i.save()


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0015_close_previously_fixed_incidents'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='services',
            field=models.ManyToManyField(blank=True, related_name='incidents', related_query_name='incident', to='statusboard.Service', verbose_name='services'),
        ),
        migrations.RunPython(copy_service_to_services_forward,
                             copy_service_to_services_backward),
        migrations.RemoveField(
            model_name='incident',
            name='service',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 20:54
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('statusboard', '0013_auto_20170220_1444'),
        ('statusboard', '0009_servicegroup_position'),
    ]

    operations = [
    ]
